<?php
namespace Battles;
abstract class Item
{
    protected $item_id;
    protected $name;
    protected $item_type;
    protected $durability;
    protected $price;
    protected $need_strength;
    protected $need_dexterity;
    protected $need_intuition;
    protected $need_endurance;
    protected $need_intelligence;
    protected $need_wisdom;
    protected $add_strength;
    protected $add_dexterity;
    protected $add_intuition;
    protected $add_endurance;
    protected $add_intelligence;
    protected $add_wisdom;
    protected $add_accuracy;
    protected $add_evasion;
    protected $add_criticals;
    protected $add_min_physical_damage;
    protected $add_max_physical_damage;
    protected $weight;
    protected $image;
    public const ITEM_TYPES_ALLOWED_IN_SLOTS = [1,2,3,4,5,6,7,8,9,10,11,12];
    public const ITEM_TYPE_HELMET = 1;
    public const ITEM_TYPE_ARMOR = 2;
    public const ITEM_TYPE_LEGS = 3;
    public const ITEM_TYPE_BOOTS = 4;
    public const ITEM_TYPE_GLOVES = 5;
    public const ITEM_TYPE_WEAPON = 6;
    public const ITEM_TYPE_SHIELD = 7;
    public const ITEM_TYPE_BELT = 8;
    public const ITEM_TYPE_RING = 9;
    public const ITEM_TYPE_AMULET = 10;
    public const ITEM_TYPE_CONSUMABLE = 20;
    public const ITEM_TYPE_OTHER = 50;
    public const ITEM_TYPE_TRASH = 100;
    private $typename;

    /**
     * Item constructor.
     *
     * @param $row
     */
    public function __construct($row)
    {
        foreach ($this as $key => $value) {
            if (isset($row[$key])) {
                $this->$key = $row[$key];
            }
        }

        switch ($this->item_type) {
            case self::ITEM_TYPE_HELMET:
                $this->typename = 'Шлем';
                break;
            case self::ITEM_TYPE_ARMOR:
                $this->typename = 'Броня';
                break;
            case self::ITEM_TYPE_LEGS:
                $this->typename = 'Поножи';
                break;
            case self::ITEM_TYPE_BOOTS:
                $this->typename = 'Сапоги';
                break;
            case self::ITEM_TYPE_GLOVES:
                $this->typename = 'Перчатки';
                break;
            case self::ITEM_TYPE_WEAPON:
                $this->typename = 'Оружие';
                break;
            case self::ITEM_TYPE_SHIELD:
                $this->typename = 'Щит';
                break;
            case self::ITEM_TYPE_BELT:
                $this->typename = 'Пояс';
                break;
            case self::ITEM_TYPE_RING:
                $this->typename = 'Кольцо';
                break;
            case self::ITEM_TYPE_AMULET:
                $this->typename = 'Амулет';
                break;
            case self::ITEM_TYPE_CONSUMABLE:
                $this->typename = 'Расходуемый предмет';
                break;
            default:
                $this->typename = 'Хлам';
        }
    }

    abstract public function printInfo();

    public function printImage()
    {
        echo <<<IMG
<img src="/i/sh/{$this->image}" class="item-wrap-normal" alt="">
IMG;
    }

    protected function wrap(int $number):string
    {
        if ($number > 0) {
            return ": <b>" . $number . "</b>";
        } else {
            return ": <b style='color: maroon;'>" . $number . "</b>";
        }
    }

    protected function printAllInfo()
    {
        $needsLines = [
            "сила" => $this->need_strength,
            "ловкость" =>$this->need_dexterity,
            "интуиция" =>$this->need_intuition,
            "выносливость" =>$this->need_endurance,
            "интеллект" =>$this->need_intelligence,
            "мудрость" =>$this->need_wisdom,
        ];
        $addsLines = [
            "Сила" => $this->add_strength,
            "Ловкость" => $this->add_dexterity,
            "Интуиция" => $this->add_intuition,
            "Выносливость" => $this->add_endurance,
            "Интеллект" => $this->add_intelligence,
            "Мудрость" => $this->add_wisdom,
            "Точность" => $this->add_accuracy,
            "Увёртливость" => $this->add_evasion,
            "Шанс крита" => $this->add_criticals,
        ];
        echo "<b>" . $this->name . "</b> (Масса: " . $this->weight . ")";
        echo "<br> Долговечность: " . $this->durability;

        echo "<br><em>{$this->typename}</em><br>";
        foreach ($needsLines as $stat => $value) {
            if ($value > 0) {
                echo "<br>Требуется $stat" . $this->wrap($value);
            }
        }
        foreach ($addsLines as $stat => $value) {
            if ($value) {
                echo "<br>$stat" . $this->wrap($value);
            }
        }
        if ($this->add_min_physical_damage && !$this->add_max_physical_damage) {
            $damage = $this->add_min_physical_damage . " - " . $this->add_min_physical_damage;
        } elseif (!$this->add_min_physical_damage && $this->add_max_physical_damage) {
            $damage = $this->add_max_physical_damage . " - " . $this->add_max_physical_damage;
        } elseif ($this->add_min_physical_damage && $this->add_max_physical_damage) {
            $damage = $this->add_min_physical_damage . " - " . $this->add_max_physical_damage;
        }
        if (isset($damage)) {
            echo "<br>Урон: " . $damage;
        }
    }
}